@@ -36,13 +36,12 @@ class WindowPreview extends St.Button {
         this._window = window;
 
         this._window.connectObject(
-            'size-changed', () => this.queue_relayout(),
-            'position-changed', () => {
-                this._updateVisible();
-                this.queue_relayout();
-            },
+            'size-changed', () => this._checkRelayout(),
+            'position-changed', () => this._checkRelayout(),
             'notify::minimized', this._updateVisible.bind(this),
+            'notify::window-type', this._updateVisible.bind(this),
             this);
+        this._updateVisible();
 
         global.display.connectObject('notify::focus-window',
             this._onFocusChanged.bind(this), this);
@@ -61,11 +60,15 @@ class WindowPreview extends St.Button {
             this.remove_style_class_name('active');
     }
 
-    _updateVisible() {
+    _checkRelayout() {
         const monitor = Main.layoutManager.findIndexForActor(this);
         const workArea = Main.layoutManager.getWorkAreaForMonitor(monitor);
-        this.visible = this._window.get_frame_rect().overlap(workArea) &&
-            this._window.window_type !== Meta.WindowType.DESKTOP &&
+        if (this._window.get_frame_rect().overlap(workArea))
+            this.queue_relayout();
+    }
+
+    _updateVisible() {
+        this.visible = this._window.window_type !== Meta.WindowType.DESKTOP &&
             this._window.showing_on_its_workspace();
     }
 }
@@ -114,6 +117,8 @@ class WorkspaceThumbnail extends St.Button {
             child: new Clutter.Actor({
                 layout_manager: new WorkspaceLayout(),
                 clip_to_allocation: true,
+                x_expand: true,
+                y_expand: true,
             }),
         });
 
@@ -258,7 +263,7 @@ export class WorkspaceIndicator extends PanelMenu.Button {
             x_expand: true,
             y_expand: true,
         });
-        this.add_actor(container);
+        this.add_child(container);
 
         let workspaceManager = global.workspace_manager;
 
@@ -271,7 +276,7 @@ export class WorkspaceIndicator extends PanelMenu.Button {
             y_expand: true,
             child: this._statusLabel,
         });
-        container.add_actor(this._statusBin);
+        container.add_child(this._statusBin);
 
         this._thumbnailsBox = new St.BoxLayout({
             style_class: 'workspaces-box',
@@ -280,7 +285,7 @@ export class WorkspaceIndicator extends PanelMenu.Button {
         });
         this._thumbnailsBox.connect('scroll-event',
             this._onScrollEvent.bind(this));
-        container.add_actor(this._thumbnailsBox);
+        container.add_child(this._thumbnailsBox);
 
         this._workspacesItems = [];
 
@@ -331,7 +336,7 @@ export class WorkspaceIndicator extends PanelMenu.Button {
         for (let i = 0; i < this._workspacesItems.length; i++) {
             this._workspacesItems[i].setOrnament(i === this._currentWorkspace
                 ? PopupMenu.Ornament.DOT
-                : PopupMenu.Ornament.NONE);
+                : PopupMenu.Ornament.NO_DOT);
         }
     }
 
@@ -377,8 +382,9 @@ export class WorkspaceIndicator extends PanelMenu.Button {
                 this._activate(item.workspaceId);
             });
 
-            if (i === this._currentWorkspace)
-                item.setOrnament(PopupMenu.Ornament.DOT);
+            item.setOrnament(i === this._currentWorkspace
+                ? PopupMenu.Ornament.DOT
+                : PopupMenu.Ornament.NO_DOT);
 
             this.menu.addMenuItem(item);
             this._workspacesItems[i] = item;
@@ -394,7 +400,7 @@ export class WorkspaceIndicator extends PanelMenu.Button {
 
         for (let i = 0; i < workspaceManager.n_workspaces; i++) {
             let thumb = new WorkspaceThumbnail(i);
-            this._thumbnailsBox.add_actor(thumb);
+            this._thumbnailsBox.add_child(thumb);
         }
         this._updateActiveThumbnail();
     }
